@@ -5,7 +5,7 @@ use warnings;
 
 use base "System::Info::BSD";
 
-our $VERSION = "0.051";
+our $VERSION = "0.052";
 
 =head1 NAME
 
@@ -27,20 +27,21 @@ sub prepare_sysinfo {
     my $system_profiler = __get_system_profiler () or
 	return $self->SUPER::prepare_sysinfo ();
 
-    if (my $kv = $system_profiler->{"System Version"}) {
+    $self->{__system_profiler} = $system_profiler;
+    if (my $kv = $system_profiler->{"system version"}) {
 	$self->{__os} =~ s{\)$}{ - $kv)};
 	}
 
-    my $model = $system_profiler->{"Machine Name"} ||
-		$system_profiler->{"Machine Model"};
+    my $model = $system_profiler->{"machine name"} ||
+		$system_profiler->{"machine model"};
 
-    my $ncpu = $system_profiler->{"Number Of CPUs"};
-    $system_profiler->{"Total Number Of Cores"} and
-	$ncpu .= " [$system_profiler->{'Total Number Of Cores'} cores]";
+    my $ncpu = $system_profiler->{"number of cpus"};
+    $system_profiler->{"total number of cores"} and
+	$ncpu .= " [$system_profiler->{'total number of cores'} cores]";
 
-    $self->{__cpu_type}  = $system_profiler->{"CPU Type"}
-	if $system_profiler->{"CPU Type"};
-    $self->{__cpu}       = "$model ($system_profiler->{'CPU Speed'})";
+    $self->{__cpu_type}  = $system_profiler->{"cpu type"}
+	if $system_profiler->{"cpu type"};
+    $self->{__cpu}       = "$model ($system_profiler->{'cpu speed'})";
     $self->{__cpu_count} = $ncpu;
 
     return $self;
@@ -49,22 +50,31 @@ sub prepare_sysinfo {
 sub __get_system_profiler {
     my $system_profiler_output = do {
 	local $^W = 0;
-	`/usr/sbin/system_profiler -detailLevel mini SPHardwareDataType SPSoftwareDataType`;
+	`/usr/sbin/system_profiler -detailLevel mini SPHardwareDataType SPSoftwareDataType 2>&1`;
 	} or return;
 
+    # From RT#97441
+    # In Yosemite the system_profiler started emitting these warnings:
+    # 2015-07-24 06:54:06.842 system_profiler[59780:1318389] platformPluginDictionary: Can\'t get X86PlatformPlugin, return value 0
+    # They seem to be harmless, but annoying.
+    # Clean them out, but then warn about others from system_profiler.
+    $system_profiler_output =~ s/^\d{4}-\d\d-\d\d .+ system_profiler\[.+?\] platformPluginDictionary: Can't get X86PlatformPlugin, return value 0$//mg;
+    warn "Unexpected warning from system_profiler:\n$1\n"
+	while $system_profiler_output =~ /^(.+system_profiler.+)/mg;
+
     my %system_profiler;
-    $system_profiler{$1} = $2
+    $system_profiler{lc $1} = $2
 	while $system_profiler_output =~ m/^\s*([\w ]+):\s+(.+)$/gm;
 
     # convert newer output from Intel core duo
     my %keymap = (
-	"Processor Name"	=> "CPU Type",
-	"Processor Speed"	=> "CPU Speed",
-	"Model Name"		=> "Machine Name",
-	"Model Identifier"	=> "Machine Model",
-	"Number Of Processors"  => "Number Of CPUs",
-	"Number of Processors"  => "Number Of CPUs",
-	"Total Number of Cores" => "Total Number Of Cores",
+	"processor name"	=> "cpu type",
+	"processor speed"	=> "cpu speed",
+	"model name"		=> "machine name",
+	"model identifier"	=> "machine model",
+	"number of processors"  => "number of cpus",
+	"number of processors"  => "number of cpus",
+	"total number of cores" => "total number of cores",
 	);
     for my $newkey (keys %keymap) {
 	my $oldkey = $keymap{$newkey};
@@ -72,10 +82,10 @@ sub __get_system_profiler {
 	    $system_profiler{$oldkey} = delete $system_profiler{$newkey};
 	}
 
-    chomp ($system_profiler{"CPU Type"} ||= `uname -m`);
-    $system_profiler{"CPU Type"} ||= "Unknown";
-    $system_profiler{"CPU Type"}   =~ s/PowerPC\s*(\w+).*/macppc$1/;
-    $system_profiler{"CPU Speed"}  =~
+    chomp ($system_profiler{"cpu type"} ||= `uname -m`);
+    $system_profiler{"cpu type"} ||= "Unknown";
+    $system_profiler{"cpu type"}   =~ s/PowerPC\s*(\w+).*/macppc$1/;
+    $system_profiler{"cpu speed"}  =~
 	s/(0(?:\.\d+)?)\s*GHz/sprintf "%d MHz", $1 * 1000/e;
 
     return \%system_profiler;
